//-----------------------------------------------------------------------------
/// Modulename : Binarize.cpp @file
///
/// Interface  : Binarize.h
///
///              Implementation of class CBinarize.
///
///  2003 Sirona Dental Systems GmbH - All Rights Reserved
///
/// Status     : Draft
///
/// Security   : -
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Standard includes
//-----------------------------------------------------------------------------
#include "stdafx.h"

//-----------------------------------------------------------------------------
// Own includes
//-----------------------------------------------------------------------------
#include "Binarize.h"

#include "isidexisng.h"  // XG object model
#include "DirectDentalEnums.h"
#include "Lcid.h"
#include "NGIIDs.h"
#include "NGObjTypes.h"

//-----------------------------------------------------------------------------
// Private definitions, -macros and -constants
//-----------------------------------------------------------------------------
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------
// Private data types
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// External references
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Module global variables
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Prototype of the private functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Implementation
//-----------------------------------------------------------------------------
IMPLEMENT_DYNCREATE(CBinarize, CCmdTarget)

BEGIN_MESSAGE_MAP(CBinarize, CCmdTarget)
   //{{AFX_MSG_MAP(CBinarize)
      // NOTE - the ClassWizard will add and remove mapping macros here.
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CBinarize, CCmdTarget)
   //{{AFX_DISPATCH_MAP(CBinarize)
   DISP_PROPERTY_EX(CBinarize, "LastParameters", GetLastParameters, SetLastParameters, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "Vendor", GetVendor, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "Description", GetDescription, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "Version", GetVersion, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "Category", GetCategory, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "MenuEntry", GetMenuEntry, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "MessageString", GetMessageString, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "TooltipText", GetTooltipText, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "FriendlyName", GetFriendlyName, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "BitmapResourceSmall", GetBitmapResourceSmall, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "BitmapResourceLarge", GetBitmapResourceLarge, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "Key", GetKey, SetNotSupported, VT_BSTR)
   DISP_PROPERTY_EX(CBinarize, "Reserved", GetReserved, SetNotSupported, VT_BSTR)
   DISP_FUNCTION(CBinarize, "Run", Run, VT_ERROR, VTS_NONE)
   //}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_IBinarize to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {54BD6209-0DC1-4F7C-A46D-B4D00AEFABB7}
static const IID IID_IBinarize =
{ 0x54bd6209, 0xdc1, 0x4f7c, { 0xa4, 0x6d, 0xb4, 0xd0, 0xa, 0xef, 0xab, 0xb7 } };

BEGIN_INTERFACE_MAP(CBinarize, CCmdTarget)
   INTERFACE_PART(CBinarize, IID_IBinarize, Dispatch)
END_INTERFACE_MAP()

// {AF126E8D-C7B4-49AB-88B3-1D515749CD5D}
IMPLEMENT_OLECREATE_EX(CBinarize, "Binarize.Binarize", 0xaf126e8d, 0xc7b4, 0x49ab, 0x88, 0xb3, 0x1d, 0x51, 0x57, 0x49, 0xcd, 0x5d)

//-----------------------------------------------------------------------------
/// Constructor
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
CBinarize::CBinarize()
{
   EnableAutomation();
   AfxOleLockApp();

   m_theImage.SetCLSID( guid );
}

//-----------------------------------------------------------------------------
/// Destructor
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
CBinarize::~CBinarize()
{
   AfxOleUnlockApp();
}

//-----------------------------------------------------------------------------
/// When the last reference for an automation object is released OnFinalRelease 
/// is called.  The base class will automatically deletes the object. Add 
/// additional cleanup required for your object before calling the base class.
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
void CBinarize::OnFinalRelease()
{
   CCmdTarget::OnFinalRelease();
}

//-----------------------------------------------------------------------------
/// Class factory for OLE server
///
/// @param     bRegister TRUE for registering
///
/// @return    Returns TRUE if successful
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BOOL CBinarize::CBinarizeFactory::UpdateRegistry( BOOL bRegister )
{
   if (bRegister)
      return AfxOleRegisterServerClass( m_clsid, m_lpszProgID, m_lpszProgID, m_lpszProgID, OAT_DISPATCH_OBJECT );
   else
      return AfxOleUnregisterClass( m_clsid, m_lpszProgID );
}

//-----------------------------------------------------------------------------
/// Retrieves the Sidexis application dispatch
///
/// @return    Application dispatch or NULL
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 16.01.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
LPDISPATCH CBinarize::GetApplication()
{
   // Retrieve ClassID
   CLSID   clsid;
   HRESULT hr = CLSIDFromProgID( L"SIDEXISNG.Application", &clsid );
   
   if (FAILED( hr ))
      return NULL;

   // Find the active object
   LPUNKNOWN pUnkSrv = NULL;

   hr = GetActiveObject( clsid, NULL, &pUnkSrv );  // returns Unknown pointer
   if (FAILED( hr ))
      return NULL;

   // Get the Dispatch interface
   LPDISPATCH pdispApp = NULL;

   hr = pUnkSrv->QueryInterface( IID_IDispatch, (void**) &pdispApp );   // returns the dispatch Interface

   if (FAILED( hr ))
   {
      (void) pUnkSrv->Release();
      return NULL;
   }
   // We don't need this anymore
   (void) pUnkSrv->Release();

   return pdispApp;
}

//-----------------------------------------------------------------------------
/// Retrieves the XG host application version
///
/// @param     strVersion    [OUT] Version as string
/// @param     nVersionMajor [OUT] Major version number
/// @param     nVersionMinor [OUT] Minor version number
///
/// @return    TRUE on success
///
/// Remarks  : Needed for compatibility modes. Due to changes in SIDEXIS XG since
///            Programming Model version 1.3 there are some compatibility issues.
///            This function is a convenient way for getting the actual host programming model version.
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 16.01.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BOOL CBinarize::GetApplicationVersion( CString& strVersion, int& nVersionMajor, int& nVersionMinor )
{
   // Initialization
   strVersion.Empty();
   nVersionMajor = 0;
   nVersionMinor = 0;

   // Get the version
   IApplication Application;

   Application.AttachDispatch( GetApplication() );

   strVersion = Application.GetVersion();

   int nPos = strVersion.Find( _T(".") );

   if (nPos == -1)
      return FALSE;

   nVersionMajor = _ttoi( strVersion.Left( nPos ) );
   nVersionMinor = _ttoi( strVersion.Right( strVersion.GetLength() - nPos - 1 ) );

   return TRUE;
}

//-----------------------------------------------------------------------------
/// Returns the Reserved string
///
/// @return    Reserved, must be empty
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 19.05.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetReserved()
{
   CString strResult;

   // Reserved, must be empty
   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Returns the last parameters for plugin execution
///
/// @return    Last parameters string
///
/// Remarks  : Due to changes at the last parameter specifications a plug-in
///            has to return different last parameter strings. Versions prior 
///            to SIDEXIS XG 1.3 have to return the whole parameter string.
///            Newer versions only need the parameters needed by the plug-in
///            itself. The rest of the parameter string which the host needs
///            will be generated and saved by SIDEXIS XG.
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetLastParameters() 
{
   CString strResult;
   CString strVersion;
   int     nVersionMajor = 0;
   int     nVersionMinor = 0;

   if (GetApplicationVersion( strVersion, nVersionMajor, nVersionMinor ))
   {
      if ((nVersionMajor == 1) && (nVersionMinor < 3))   // version < 1.3?
         strResult = m_theImage.GetParamsCompatible(); // compatibility mode
      else
         strResult = m_theImage.GetParams();
   }
   else
   {
      strResult = m_theImage.GetParams();
   }

   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Sets the last parameters for plugin execution
///
/// @param     lpszNewValue [IN] Last parameters
///
/// Remarks  : Due to changes at the last parameter specifications a plug-in
///            has to parse different last parameter strings.
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
void CBinarize::SetLastParameters( LPCTSTR lpszNewValue )
{
   CString strVersion;
   int     nVersionMajor = 0;
   int     nVersionMinor = 0;

   if (GetApplicationVersion( strVersion, nVersionMajor, nVersionMinor ))
   {
      if ((nVersionMajor == 1) && (nVersionMinor < 3))   // version < 1.3?
         (void) m_theImage.SetParamsCompatible( lpszNewValue ); // compatibility mode
      else
         (void) m_theImage.SetParams( lpszNewValue );
   }
   else
   {
      (void) m_theImage.SetParams( lpszNewValue );
   }
}

//-----------------------------------------------------------------------------
/// Returns the vendor string
///
/// @return    Vendor string
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetVendor() 
{
   CString strResult;
   (void) strResult.LoadString( IDS_COMPANY );
   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Returns the localized plugin description
///
/// @return    Plugin description
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetDescription() 
{
   CString      strResult;
   IApplication Application;

   Application.AttachDispatch( GetApplication() );

   switch (Application.GetLanguage())
   {
      case LANGUAGE_GERMAN:
         (void) strResult.LoadString( IDS_DESCRIPTION_GERMAN );
         break;

      case LANGUAGE_ENGLISH_US:
      default:
         (void) strResult.LoadString( IDS_DESCRIPTION_ENGLISH );
         break;
   }

   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Returns plugin version
///
/// @return    Plugin version string
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetVersion() 
{
   CString strResult;

   (void) strResult.LoadString( IDS_VERSION );
   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Returns plugin category
///
/// @return    Plugin category
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetCategory() 
{
   CString strResult = CATEGORY_FILTER_XRAY;

   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Return the localized menu entry string
///
/// @return    Menu entry string
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetMenuEntry() 
{
   CString      strResult;
   IApplication Application;

   Application.AttachDispatch( GetApplication() );

   switch (Application.GetLanguage())
   {
      case LANGUAGE_GERMAN:
         (void) strResult.LoadString( IDS_MENUENTRY_GERMAN );
         break;

      case LANGUAGE_ENGLISH_US:
      default:
         (void) strResult.LoadString( IDS_MENUENTRY_ENGLISH );
         break;
   }

   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Returns the localized status bar string
///
/// @return    Status bar string
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetMessageString() 
{
   CString     strResult;
   IApplication Application;

   Application.AttachDispatch( GetApplication() );

   switch (Application.GetLanguage())
   {
      case LANGUAGE_GERMAN:
         (void) strResult.LoadString( IDS_STATUSBAR_GERMAN );
         break;

      case LANGUAGE_ENGLISH_US:
      default:
         (void) strResult.LoadString( IDS_STATUSBAR_ENGLISH );
         break;
   }

   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Return the localized tooltip text
///
/// @return    Tooltip text
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetTooltipText() 
{
   CString     strResult;
   IApplication Application;

   Application.AttachDispatch( GetApplication() );

   switch (Application.GetLanguage())
   {
      case LANGUAGE_GERMAN:
         (void) strResult.LoadString( IDS_TOOLTIP_GERMAN );
         break;

      case LANGUAGE_ENGLISH_US:
      default:
         (void) strResult.LoadString( IDS_TOOLTIP_ENGLISH );
         break;
   }
   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Returns the localized friendly name
///
/// @return    Friendly name
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetFriendlyName() 
{
   CString     strResult;
   IApplication Application;

   Application.AttachDispatch( GetApplication() );

   switch (Application.GetLanguage())
   {
      case LANGUAGE_GERMAN:
         (void) strResult.LoadString( IDS_FRIENDLYNAME_GERMAN );
         break;

      case LANGUAGE_ENGLISH_US:
      default:
         (void) strResult.LoadString( IDS_FRIENDLYNAME_ENGLISH );
         break;
   }
   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Returns the small bitmap resource for toolbar bitmaps
///
/// @return    Bitmap resource name
///
/// Remarks  : Due to problems with Windows 98 when loading outprocess (.exe) PlugIns with
///            LoadLibrary() there has to be a mechanism to load bitmaps from file directly.
///            For compatibility to older SIDEXIS XG versions the resource ID is returned.
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetBitmapResourceSmall() 
{
   CString strVersion;
   int     nVersionMajor, nVersionMinor;
   CString strResult;

   if (GetApplicationVersion( strVersion, nVersionMajor, nVersionMinor ))
   {
      if ((nVersionMajor == 1) && (nVersionMinor < 3))   // version < 1.3?
         strResult = _T("IDB_SMALL");
      else
         strResult = _T("idb_small.bmp");
   }
   else
      strResult = _T("idb_small.bmp");
   
   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Returns the large bitmap resource for toolbar bitmaps
///
/// @return    Bitmap resource name
///
/// Remarks  : Due to problems with Windows 98 when loading outprocess (.exe) PlugIns with
///            LoadLibrary() there has to be a mechanism to load bitmaps from file directly.
///            For compatibility to older SIDEXIS XG versions the resource ID is returned.
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetBitmapResourceLarge() 
{
   CString strVersion;
   int     nVersionMajor, nVersionMinor;
   CString strResult;

   if (GetApplicationVersion( strVersion, nVersionMajor, nVersionMinor ))
   {
      if (nVersionMajor == 1 && nVersionMinor < 3)
         strResult = _T("IDB_LARGE");
      else
         strResult = _T("idb_large.bmp");
   }
   else
      strResult = _T("idb_large.bmp");

   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Reserved
///
/// @return    Reserved, must be empty
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BSTR CBinarize::GetKey() 
{
   CString strResult;

   // Reserved, must be empty
   return strResult.AllocSysString();
}

//-----------------------------------------------------------------------------
/// Plugin execution
///
/// @return    OLE Errorcode
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
SCODE CBinarize::Run()
{
   IApplication Application;
   IExam        Exam;
   LPDISPATCH   pdispExam  = NULL;
   LPDISPATCH   pdispImage = NULL;
   CString      strStatusMsgWorking;
   CString      strStatusMsgReady;

   Application.AttachDispatch( GetApplication() );

   switch (Application.GetLanguage())
   {
      case LANGUAGE_GERMAN:
         (void) strStatusMsgWorking.LoadString( IDS_WORKING_GERMAN );
         (void) strStatusMsgReady.LoadString( IDS_READY_GERMAN );
         break;

      case LANGUAGE_ENGLISH_US:
      default:
         (void) strStatusMsgWorking.LoadString( IDS_WORKING_ENGLISH );
         (void) strStatusMsgReady.LoadString( IDS_READY_ENGLISH );
         break;
   }
   Application.SetStatusBar( strStatusMsgWorking );

   pdispExam = Application.GetActiveExam();
   if (!pdispExam)
   {
      OutputDebugString( _T("CBinarize::Run() No active examination\n") );
      Application.SetStatusBar( strStatusMsgReady );
      return E_INVALIDARG;
   }

   Exam.AttachDispatch( pdispExam );

   pdispImage = Exam.GetActiveImage();
   if (!pdispImage)
   {
      OutputDebugString( _T("CBinarize::Run() No active image\n") );
      Application.SetStatusBar( strStatusMsgReady );
      return E_INVALIDARG;
   }

   if (!m_theImage.FromSidexis( pdispImage ))
   {
      OutputDebugString( _T("CBinarize::Run() Couldn't get image pixel map\n") );
      Application.SetStatusBar( strStatusMsgReady );
      return E_FAIL;
   }

   // Do the work!
   if (!m_theImage.GetBinarizeParameters())
   {
      OutputDebugString( _T("CBinarize::Run() Couldn't retrieve parameters\n") );
      Application.SetStatusBar( strStatusMsgReady );
      return S_FALSE;
   }

   if (!m_theImage.Binarize())
   {
      OutputDebugString( _T("CBinarize::Run() Couldn't filter image pixel map\n") );
      Application.SetStatusBar( strStatusMsgReady );
      return E_FAIL;
   }

   (void) m_theImage.ToSidexis( pdispImage );

   (void) pdispImage->Release();

   Exam.Invalidate();

   Application.SetStatusBar( strStatusMsgReady );

   return S_OK;
}
