//-----------------------------------------------------------------------------
/// Modulename : customimage.cpp @file
///
/// Interface  : customimage.h
///
///              This is the imaging class for customization by yourself.
///
///  2003 Sirona Dental Systems GmbH - All Rights Reserved
///
/// Status     : Draft
///
/// Security   : -
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Standard includes
//-----------------------------------------------------------------------------
#include "stdafx.h"

//-----------------------------------------------------------------------------
// Own includes
//-----------------------------------------------------------------------------
#include "CustomImage.h"
#include "BinarizeDlg.h"

//-----------------------------------------------------------------------------
// Private definitions, -macros and -constants
//-----------------------------------------------------------------------------
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------
// Private data types
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// External references
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Module global variables
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Prototype of the private functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Implementation
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Constructor
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
CCustomImage::CCustomImage()
{
}

//-----------------------------------------------------------------------------
/// Destructor
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
CCustomImage::~CCustomImage()
{
}

//-----------------------------------------------------------------------------
/// This dialog gets the parameter for the binarize filter
///
/// @return    Returns TRUE if successful
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BOOL CCustomImage::GetBinarizeParameters()
{
   //--------------------------------------------------------------------------
   // Get the Parameters
   //--------------------------------------------------------------------------
   if (m_rgstrParameters.IsEmpty())
   {
      CBinarizeDlg dlg;

      // First call
      if (dlg.DoModal() != IDOK)
         return FALSE;

      // Save the Parameters as String for Redo later on
      m_rgstrParameters.AddTail( dlg.GetThreshold() );
   }

   return TRUE;
}

//-----------------------------------------------------------------------------
/// This is the binarize filter function
///
/// @return    Returns TRUE if successful
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
BOOL CCustomImage::Binarize()
{
   // This filter only supports grayscale images
   if (GetBitsPerPixel() != 8 || !m_hBitmap)
      return FALSE;

   // Lookup parameter 1
   if (m_rgstrParameters.GetCount() < 1)
      return FALSE;

   // This sample only uses one parameter. If you would like to use more, you should map them through this string list correctly
   POSITION pos = m_rgstrParameters.FindIndex( 0 );

   if (!pos)
      return FALSE;

   CString strValue = m_rgstrParameters.GetAt( pos );

   UINT uiParameter1 = _ttol( strValue );  // Convert to what you need

   // Check parameters
   if (uiParameter1 > 255)
      return FALSE;

   // filter kernel
   CBitmap* pBitmap = CBitmap::FromHandle( m_hBitmap );

   CDC dcMemory;
   dcMemory.CreateCompatibleDC( NULL );

   CBitmap* pOld = dcMemory.SelectObject( pBitmap );

   for (int nY = m_rcROI.top; nY < m_rcROI.top + m_rcROI.Height(); nY++)
   {
      for (int nX = m_rcROI.left; nX < m_rcROI.left + m_rcROI.Width(); nX++)
      {
         COLORREF cr = dcMemory.GetPixel( nX, nY );

         if (cr < RGB( uiParameter1, uiParameter1, uiParameter1 ))
            dcMemory.SetPixelV( nX, nY, RGB( 0, 0, 0 ) );
         else
            dcMemory.SetPixelV( nX, nY, RGB( 255, 255, 255 ) );
      }
   }

   dcMemory.SelectObject( pOld );

   // well done...
   return TRUE;
}
