//-----------------------------------------------------------------------------
/// Headername  : fwimage.h @file
///
///               Image framework
///
///  2003 Sirona Dental Systems GmbH - All Rights Reserved
///
/// Status      : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
#ifndef __FWIMAGE_H
#define __FWIMAGE_H

//-----------------------------------------------------------------------------
// Standard includes
//-----------------------------------------------------------------------------
#include <afxtempl.h>         // MFC Template classes

//-----------------------------------------------------------------------------
// Own includes
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Definitions, macros und constants
//-----------------------------------------------------------------------------
#if _MSC_VER > 1000 // VC++ Version > 4.0
#pragma once // open only once during build
#endif // _MSC_VER > 1000

//-----------------------------------------------------------------------------
// Data types
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Prototype of the public functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Classname  : CImage : public CObject
///
/// Image base wrapper class. Provides a framework for the passing of data 
/// between the plugin and the main application.
//-----------------------------------------------------------------------------
class CImage : public CObject
{
   //---------------------------------------------------------------------------
   // Member functions
   //---------------------------------------------------------------------------
public:
            CImage();
   virtual ~CImage();

   void           Clear();

   BOOL           FromSidexis(      LPDISPATCH     pdispImage        );
   BOOL           ToSidexis(        LPDISPATCH     pdispImage        );

   UINT           GetWidth() const         { return m_pbmi->bmiHeader.biWidth; }
   UINT           GetHeight() const        { return m_pbmi->bmiHeader.biHeight; }
   UINT           GetBitsPerPixel() const  { return m_pbmi->bmiHeader.biBitCount; }
   DWORD          GetSize() const          { return m_pbmi->bmiHeader.biSizeImage; }

   UINT           GetBytesPerPixel() const { return (m_pbmi->bmiHeader.biBitCount >> 3); }
   UINT           GetByteWidth() const     { return m_pbmi->bmiHeader.biWidth * GetBytesPerPixel(); }
   BITMAPINFO*    GetBitmapInfo()          { return m_pbmi; }

   const CRect&   GetROI() const           { return m_rcROI; }

   HBITMAP        GetBitmap()              { return m_hBitmap; }

   BOOL           SetParams( const CString& strFilterParameters );
   CString        GetParams();

   BOOL           SetParamsCompatible( const CString& strParameters );
   CString        GetParamsCompatible();

   void           SetCLSID( CLSID clsid )  { m_clsid = clsid; }

   //---------------------------------------------------------------------------
   // Member variables                                       - Short description
   //---------------------------------------------------------------------------
protected:
   HBITMAP        m_hBitmap;           ///< Handle of device independent Bitmap
   BITMAPINFO*    m_pbmi;              ///< Bitmap info
   CRect          m_rcROI;             ///< ROI, if one has been set
   
   CStringList    m_rgstrParameters;   ///< This is the list of n Parameters

   CLSID          m_clsid;             ///< COM CLSID
};

#endif //__FWIMAGE_H
