#include "stdafx.h"
#include "About.h"
#include "objectmodel.h"
#include "PluginDialog.h"
#include "ISidexisNG.h"
#include "NGObjTypes.h"
#include "NGIIDs.h"
#include "LCID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(CPluginDialog, CDialog)
   //{{AFX_MSG_MAP(CPluginDialog)
	ON_BN_CLICKED(IDC_UPDATE, OnUpdate)
	ON_BN_CLICKED(IDC_ABOUT, OnAbout)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CPluginDialog::CPluginDialog(CWnd* pParent /*=NULL*/)
   : CDialog(CPluginDialog::IDD, pParent)
{
   //{{AFX_DATA_INIT(CPluginDialog)
      // NOTE: the ClassWizard will add member initialization here
   //}}AFX_DATA_INIT
}

void CPluginDialog::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   //{{AFX_DATA_MAP(CPluginDialog)
	DDX_Control(pDX, IDC_OBJECTMODELTREE, m_ctlObjectModel);
	//}}AFX_DATA_MAP
}

BOOL CPluginDialog::OnInitDialog() 
{
   CDialog::OnInitDialog();

   DWORD dwStyle = m_ctlObjectModel.GetStyle();
   dwStyle = dwStyle | TVS_FULLROWSELECT | TVS_HASBUTTONS | TVS_HASLINES | TVS_LINESATROOT | TVS_SHOWSELALWAYS;

   (void) m_ctlObjectModel.ModifyStyle( 0, dwStyle );

   UpdateModel();

   return TRUE;
}

LPDISPATCH CPluginDialog::GetApplication()
{
   HRESULT      hr = S_OK;
   CLSID        clsid;
   IApplication ISidexis;
   LPUNKNOWN    pUnkSrv  = NULL;
   LPDISPATCH   pdispApp = NULL;

   // Retrieve ClassID
   hr = CLSIDFromProgID( L"SIDEXISNG.Application", &clsid );
   
   if (FAILED( hr ))
      return NULL;

   // Find the active object
   hr = GetActiveObject( clsid, NULL, &pUnkSrv );  // returns Unknown pointer

   if (FAILED( hr ))
      return NULL;

   // Get the Dispatch interface
   hr = pUnkSrv->QueryInterface( IID_IDispatch, (void**) &pdispApp );   // returns the dispatch Interface

   if (FAILED( hr ))
   {
      (void) pUnkSrv->Release();
      return NULL;
   }

   // We don't need this anymore
   (void) pUnkSrv->Release();

   return pdispApp;
}

void CPluginDialog::UpdateModel()
{
   // Empty the list
   (void) m_ctlObjectModel.DeleteAllItems();

   // Insert Root
   HTREEITEM htRoot = m_ctlObjectModel.InsertItem( _T("SIDEXIS XG object tree"), 0, 0, TVI_ROOT, TVI_LAST );

   // Iterating PM and listing all objects
   LPDISPATCH pdispXG = GetApplication();
   if (pdispXG)
   {
      IApplication SidexisXG;
      SidexisXG.AttachDispatch( GetApplication(), FALSE );

      HTREEITEM htApp = m_ctlObjectModel.InsertItem( _T("Application (Version: ") + SidexisXG.GetVersion() + _T(", Language: ") + GetLanguage( SidexisXG.GetLanguage() ) + _T(")"), 0, 0, htRoot, TVI_LAST );

      LPDISPATCH pdispActiveExam = SidexisXG.GetActiveExam(); 
      if (pdispActiveExam)
      {
         IExam Exam;
         Exam.AttachDispatch( pdispActiveExam, FALSE );

         CString strExamName = Exam.GetName().IsEmpty() ? _T("Unnamed") : Exam.GetName();
         HTREEITEM htExam = m_ctlObjectModel.InsertItem( _T("Exam (") + strExamName + _T(")"), 0, 0, htApp, TVI_LAST );

         // Patient
         LPDISPATCH pdispPatient = Exam.GetPatient();
         if (pdispPatient)
         {
            IPatient Patient;
            Patient.AttachDispatch( pdispPatient, FALSE );

            HTREEITEM htPatient = m_ctlObjectModel.InsertItem( _T("Patient (") + Patient.GetLastname() + _T(", ") + Patient.GetFirstname() + _T(")"), 0, 0, htExam, TVI_LAST );
         }

         // TakeFrames
         LPDISPATCH pdispTakeFrames = Exam.GetTakeFrames();
         if (pdispTakeFrames)
         {
            ITakeFrames TakeFrames;
            TakeFrames.AttachDispatch( pdispTakeFrames );

            for (long lCount = 0; lCount < TakeFrames.GetCount(); lCount++ )
            {
               LPDISPATCH pdispTakeFrame = TakeFrames.GetItem( lCount );
               if (pdispTakeFrame)
               {
                  // Frame
                  ITakeFrame TakeFrame;
                  TakeFrame.AttachDispatch( pdispTakeFrame, FALSE );

                  HTREEITEM htFrame = m_ctlObjectModel.InsertItem( _T("TakeFrame (") + TakeFrame.GetName() + _T(")"), 0, 0, htExam, TVI_LAST );

                  LPDISPATCH pdispImage = TakeFrame.GetImage();
                  if (pdispImage)
                  {
                     // Image
                     IImage Image;
                     Image.AttachDispatch( pdispImage, FALSE );

                     HTREEITEM htImage = m_ctlObjectModel.InsertItem( _T("Image (") + Image.GetName() + _T(")"), 0, 0, htFrame, TVI_LAST );

                     // ROI
                     LPDISPATCH pdispROI = Image.GetRoi();
                     if (pdispROI)
                     {
                        IROI ROI;
                        ROI.AttachDispatch( pdispROI );

                        long lLeft, lTop, lWidth, lHeight;
                        ROI.GetPosition( &lLeft, &lTop, &lWidth, &lHeight );

                        CString strPositionROI;
                        strPositionROI.Format( _T("l=%d, t=%d, w=%d, h=%d"), lLeft, lTop, lWidth, lHeight );

                        HTREEITEM htImageROI = m_ctlObjectModel.InsertItem( _T("ROI (") + strPositionROI + _T(")"), 0, 0, htImage, TVI_LAST );
                     }

                     // LUT
                     LPDISPATCH pdispLUT = Image.GetLut();
                     if (pdispLUT)
                     {
                        ILUT LUT;
                        LUT.AttachDispatch( pdispLUT );
                     }

                     // Diagnoses
                     DumpDiagnoses( Image.GetDiagnoses(), htImage );

                     // Angles
                     DumpAngles( Image.GetAngles(), htImage );

                     // Cuts
                     DumpCuts( Image.GetCuts(), htImage );

                     // Lengths
                     DumpLengths( Image.GetLengths(), htImage );

                     // Image objects
                     DumpObjects( Image.GetObjects(), htImage );
                  }
               }
            }
         }

         // Exam objects
         DumpObjects( Exam.GetObjects(), htExam );

         // Selection objects
         DumpSelectionObjects( Exam.GetSelectionObjects(), htExam );
      }
   }

   m_ctlObjectModel.Expand( m_ctlObjectModel.GetRootItem(), TVE_EXPAND );
   m_ctlObjectModel.Select( m_ctlObjectModel.GetRootItem(), TVGN_CARET );
}

void CPluginDialog::DumpSelectionObjects( LPDISPATCH pdispSelectionObjects, HTREEITEM htParent )
{
   if (!pdispSelectionObjects)
      return;

   ISelectionObjects SelectionObjects;
   SelectionObjects.AttachDispatch( pdispSelectionObjects );

   CString strCount;
   strCount.Format( _T("%d"), SelectionObjects.GetCount() );

   HTREEITEM htSelectionObjects = m_ctlObjectModel.InsertItem( _T("Selected Objects (") + strCount + _T(")"), 0, 0, htParent, TVI_LAST );

   for (long lSelectionObjects = 0; lSelectionObjects < SelectionObjects.GetCount(); lSelectionObjects++)
      DumpObject( SelectionObjects.GetItem( lSelectionObjects ), htSelectionObjects );
}

void CPluginDialog::DumpObjects( LPDISPATCH pdispObjects, HTREEITEM htParent )
{
   if (!pdispObjects)
      return;

   IObjects Objects;
   Objects.AttachDispatch( pdispObjects );

   CString strCount;
   strCount.Format( _T("%d"), Objects.GetCount() );

   HTREEITEM htObjects = m_ctlObjectModel.InsertItem( _T("Objects (") + strCount + _T(")"), 0, 0, htParent, TVI_LAST );

   for (long lObjects = 0; lObjects < Objects.GetCount(); lObjects++)
      DumpObject( Objects.GetItem( lObjects ), htObjects );
}

void CPluginDialog::DumpObject( LPDISPATCH pdispObject, HTREEITEM htParent )
{
   if (!pdispObject)
      return;

   IObject Object;
   Object.AttachDispatch( pdispObject );

   HTREEITEM htObject = m_ctlObjectModel.InsertItem( _T("Object (") + GetObjectType( Object.GetType() ) + _T(")"), 0, 0, htParent, TVI_LAST );

   switch (Object.GetType())
   {
      case NGArrow:
         DumpArrow( Object.GetTypedObject(), htObject );
         break;

      case NGSymbol:
         DumpSymbol( Object.GetTypedObject(), htObject );
         break;

      case NGRect:
         DumpRect( Object.GetTypedObject(), htObject );
         break;

      case NGEllipse:
         DumpEllipse( Object.GetTypedObject(), htObject );
         break;

      case NGCut:
         DumpCut( Object.GetTypedObject(), htObject );
         break;

      default:
         break;
   }
}

void CPluginDialog::DumpDiagnoses( LPDISPATCH pdispDiagnoses, HTREEITEM htParent )
{
   if (!pdispDiagnoses)
      return;

   IDiagnoses Diagnoses;
   Diagnoses.AttachDispatch( pdispDiagnoses );

   CString strDiagnosesCount;
   strDiagnosesCount.Format( _T("%d"), Diagnoses.GetCount() );

   HTREEITEM htDiagnoses = m_ctlObjectModel.InsertItem( _T("Diagnoses (") + strDiagnosesCount + _T(")"), 0, 0, htParent, TVI_LAST );

   for (long lCountDiagnoses = 0; lCountDiagnoses < Diagnoses.GetCount(); lCountDiagnoses++)
      DumpDiagnosis( Diagnoses.GetItem( lCountDiagnoses ), htDiagnoses );
}

void CPluginDialog::DumpDiagnosis( LPDISPATCH pdispDiagnosis, HTREEITEM htParent )
{
   if (!pdispDiagnosis)
      return;

   IDiagnosis Diagnosis;
   Diagnosis.AttachDispatch( pdispDiagnosis, FALSE );

   HTREEITEM htDiagnosis = m_ctlObjectModel.InsertItem( _T("Diagnosis (") + Diagnosis.GetComment() + _T(")"), 0, 0, htParent, TVI_LAST );

   DumpPoints( Diagnosis.GetPoints(), htDiagnosis );
}

void CPluginDialog::DumpAngles( LPDISPATCH pdispAngles, HTREEITEM htParent )
{
   if (!pdispAngles)
      return;

   IAngles Angles;
   Angles.AttachDispatch( pdispAngles );

   CString strAnglesCount;
   strAnglesCount.Format( _T("%d"), Angles.GetCount() );

   HTREEITEM htAngles = m_ctlObjectModel.InsertItem( _T("Angles (") + strAnglesCount + _T(")"), 0, 0, htParent, TVI_LAST );

   for (long lCountAngles = 0; lCountAngles < Angles.GetCount(); lCountAngles++)
      DumpAngle( Angles.GetItem( lCountAngles ), htAngles );
}

void CPluginDialog::DumpAngle( LPDISPATCH pdispAngle, HTREEITEM htParent )
{
   if (!pdispAngle)
      return;

   IAngle Angle;
   Angle.AttachDispatch( pdispAngle, FALSE );

   HTREEITEM htAngle = m_ctlObjectModel.InsertItem( _T("Angle"), 0, 0, htParent, TVI_LAST );

   DumpPoints( Angle.GetPoints(), htAngle );
}

void CPluginDialog::DumpCuts( LPDISPATCH pdispCuts, HTREEITEM htParent )
{
   if (!pdispCuts)
      return;

   ICuts Cuts;
   Cuts.AttachDispatch( pdispCuts );

   CString strCutsCount;
   strCutsCount.Format( _T("%d"), Cuts.GetCount() );

   HTREEITEM htCuts = m_ctlObjectModel.InsertItem( _T("Cuts (") + strCutsCount + _T(")"), 0, 0, htParent, TVI_LAST );

   for (long lCountCuts = 0; lCountCuts < Cuts.GetCount(); lCountCuts++)
      DumpCut( Cuts.GetItem( lCountCuts ), htCuts );
}

void CPluginDialog::DumpCut( LPDISPATCH pdispCut, HTREEITEM htParent )
{
   if (!pdispCut)
      return;

   ICut Cut;
   Cut.AttachDispatch( pdispCut, FALSE );

   HTREEITEM htCut = m_ctlObjectModel.InsertItem( _T("Cut"), 0, 0, htParent, TVI_LAST );

   DumpPoints( Cut.GetPoints(), htCut );
}

void CPluginDialog::DumpLengths( LPDISPATCH pdispLengths, HTREEITEM htParent )
{
   if (!pdispLengths)
      return;

   ILengths Lengths;
   Lengths.AttachDispatch( pdispLengths );

   CString strLengthsCount;
   strLengthsCount.Format( _T("%d"), Lengths.GetCount() );

   HTREEITEM htLengths = m_ctlObjectModel.InsertItem( _T("Lengths (") + strLengthsCount + _T(")"), 0, 0, htParent, TVI_LAST );

   for (long lCountLengths = 0; lCountLengths < Lengths.GetCount(); lCountLengths++)
      DumpLength( Lengths.GetItem( lCountLengths ), htLengths );
}

void CPluginDialog::DumpLength( LPDISPATCH pdispLength, HTREEITEM htParent )
{
   if (!pdispLength)
      return;

   ILength Length;
   Length.AttachDispatch( pdispLength, FALSE );

   HTREEITEM htLength = m_ctlObjectModel.InsertItem( _T("Length"), 0, 0, htParent, TVI_LAST );

   DumpPoints( Length.GetPoints(), htLength );
}

void CPluginDialog::DumpArrow( LPDISPATCH pdispArrow, HTREEITEM htParent )
{
   if (!pdispArrow)
      return;

   IArrow Arrow;
   Arrow.AttachDispatch( pdispArrow );

   long lX, lY;

   HTREEITEM htArrowBegin = m_ctlObjectModel.InsertItem( _T("Object attached to begin"), 0, 0, htParent, TVI_LAST );

   LPDISPATCH pdispObjectAtBegin = Arrow.GetObjectAttachedToBegin( &lX, &lY );
   if (pdispObjectAtBegin)
   {
      switch (GetObjectTypeFromDispatch( pdispObjectAtBegin ))
      {
         case NGImage:
            {
               IImage ImageArrow;
               ImageArrow.AttachDispatch( pdispObjectAtBegin );

               HTREEITEM htObjectAtBegin = m_ctlObjectModel.InsertItem( _T("Image (") + ImageArrow.GetName() + _T(")"), 0, 0, htArrowBegin, TVI_LAST );
            }
            break;

         case NGDiagnosis:
            {
               IDiagnosis DiagnosisArrow;
               DiagnosisArrow.AttachDispatch( pdispObjectAtBegin );

               HTREEITEM htObjectAtBegin = m_ctlObjectModel.InsertItem( _T("Diagnosis (") + DiagnosisArrow.GetComment() + _T(")"), 0, 0, htArrowBegin, TVI_LAST );
            }
            break;

         default:
            HTREEITEM htObjectAtBegin = m_ctlObjectModel.InsertItem( _T("Unknown"), 0, 0, htArrowBegin, TVI_LAST );
            break;
      }
   }

   HTREEITEM htArrowEnd = m_ctlObjectModel.InsertItem( _T("Object attached to end"), 0, 0, htParent, TVI_LAST );

   LPDISPATCH pdispObjectAtEnd = Arrow.GetObjectAttachedToEnd( &lX, &lY );
   if (pdispObjectAtEnd)
   {
      switch (GetObjectTypeFromDispatch( pdispObjectAtEnd ))
      {
         case NGImage:
            {
               IImage ImageArrow;
               ImageArrow.AttachDispatch( pdispObjectAtEnd );

               HTREEITEM htObjectAtEnd = m_ctlObjectModel.InsertItem( _T("Image (") + ImageArrow.GetName() + _T(")"), 0, 0, htArrowEnd, TVI_LAST );
            }
            break;

         case NGDiagnosis:
            {
               IDiagnosis DiagnosisArrow;
               DiagnosisArrow.AttachDispatch( pdispObjectAtEnd );

               HTREEITEM htObjectAtEnd = m_ctlObjectModel.InsertItem( _T("Diagnosis (") + DiagnosisArrow.GetComment() + _T(")"), 0, 0, htArrowEnd, TVI_LAST );
            }
            break;

         default:
            HTREEITEM htObjectAtEnd = m_ctlObjectModel.InsertItem( _T("Unknown"), 0, 0, htArrowEnd, TVI_LAST );
            break;
      }
   }
}

void CPluginDialog::DumpSymbol( LPDISPATCH pdispSymbol, HTREEITEM htParent )
{
   if (!pdispSymbol)
      return;

   ISymbol Symbol;
   Symbol.AttachDispatch( pdispSymbol );

   long lLeft, lTop, lWidth, lHeight;
   Symbol.GetPosition( &lLeft, &lTop, &lWidth, &lHeight );

   CString strSymbol;
   strSymbol.Format( _T("Label: %s, Type: %s, Position: (%d, %d, %d, %d)"), Symbol.GetLabel(), Symbol.GetType(), lLeft, lTop, lWidth, lHeight );

   HTREEITEM htSymbol = m_ctlObjectModel.InsertItem( strSymbol, 0, 0, htParent, TVI_LAST );
}

void CPluginDialog::DumpRect( LPDISPATCH pdispRect, HTREEITEM htParent )
{
   if (!pdispRect)
      return;

   IRect Rect;
   Rect.AttachDispatch( pdispRect );

   long lLeft, lTop, lWidth, lHeight;
   Rect.GetPosition( &lLeft, &lTop, &lWidth, &lHeight );

   CString strRect;
   strRect.Format( _T("Position: (%d, %d, %d, %d)"), lLeft, lTop, lWidth, lHeight );

   HTREEITEM htRect = m_ctlObjectModel.InsertItem( strRect, 0, 0, htParent, TVI_LAST );
}

void CPluginDialog::DumpEllipse( LPDISPATCH pdispEllipse, HTREEITEM htParent )
{
   if (!pdispEllipse)
      return;

   IEllipse Ellipse;
   Ellipse.AttachDispatch( pdispEllipse );

   long lLeft, lTop, lWidth, lHeight;
   Ellipse.GetPosition( &lLeft, &lTop, &lWidth, &lHeight );

   CString strEllipse;
   strEllipse.Format( _T("Position: (%d, %d, %d, %d)"), lLeft, lTop, lWidth, lHeight );

   HTREEITEM htEllipse = m_ctlObjectModel.InsertItem( strEllipse, 0, 0, htParent, TVI_LAST );
}

void CPluginDialog::DumpPoints( LPDISPATCH pdispPoints, HTREEITEM htParent )
{
   if (!pdispPoints)
      return;

   IPoints Points;
   Points.AttachDispatch( pdispPoints );

   HTREEITEM htPoints = m_ctlObjectModel.InsertItem( _T("Points"), 0, 0, htParent, TVI_LAST );

   for (long lPoints = 0; lPoints < Points.GetCount(); lPoints++)
      DumpPoint( Points.GetItem( lPoints ), htPoints );
}

void CPluginDialog::DumpPoint( LPDISPATCH pdispPoint, HTREEITEM htParent )
{
   if (!pdispPoint)
      return;

   IPoint Point;
   Point.AttachDispatch( pdispPoint );

   long lX, lY;

   Point.Get( &lX, &lY );

   CString strPoint;
   strPoint.Format( _T("%d, %d"), lX, lY );

   HTREEITEM htPoint = m_ctlObjectModel.InsertItem( _T("Point (") + strPoint + _T(")"), 0, 0, htParent, TVI_LAST );
}

CString CPluginDialog::GetObjectType( long lObjectType )
{
   CString strObjectType;

   switch (lObjectType)
   {
      case NGUnknown:
         strObjectType = _T("NGUnknown");
         break;

      case NGTake:
         strObjectType = _T("NGTake");
         break;

      case NGDiagnosis:
         strObjectType = _T("NGDiagnosis");
         break;

      case NGArrow:
         strObjectType = _T("NGArrow");
         break;

      case NGRoi:
         strObjectType = _T("NGRoi");
         break;

      case NGLength:
         strObjectType = _T("NGLength");
         break;

      case NGText:
         strObjectType = _T("NGText");
         break;

      case NGAngle:
         strObjectType = _T("NGAngle");
         break;

      case NGCut:
         strObjectType = _T("NGCut");
         break;

      case NGImage:
         strObjectType = _T("NGImage");
         break;

      case NGExam:
         strObjectType = _T("NGExam");
         break;

      case NGPolyline:
         strObjectType = _T("NGPolyline");
         break;

      case NGRect:
         strObjectType = _T("NGRect");
         break;

      case NGEllipse:
         strObjectType = _T("NGEllipse");
         break;

      case NGSymbol:
         strObjectType = _T("NGSymbol");
         break;

      case NGNone:
      default:
         strObjectType = _T("NGNone");
         break;
   }

   return strObjectType;
}

CString CPluginDialog::GetMeasureUnit( long lMeasureUnit )
{
   CString strResult;

   switch (lMeasureUnit)
   {
      case NGPixel:
         strResult = _T("Pixel");
         break;

      case NGMicrometer:
         strResult = _T("Micrometer");
         break;

      case NGDegree:
         strResult = _T("Degree");
         break;

      case NGPercent:
         strResult = _T("Percent");
         break;

      default:
         break;
   }

   return strResult;
}

long CPluginDialog::GetObjectTypeFromDispatch( LPDISPATCH pdisp )
{
   LPUNKNOWN pUnk = NULL;
   long lResult = 0;

   if( SUCCEEDED( pdisp->QueryInterface( IID_ITakeFrame, (void**) &pUnk ) ) )
   {
      lResult = NGTake;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IDiagnosis, (void**) &pUnk ) ) )
   {
      lResult = NGDiagnosis;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IArrow, (void**) &pUnk ) ) )
   {
      lResult = NGArrow;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IROI, (void**) &pUnk ) ) )
   {
      lResult = NGRoi;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_ILength, (void**) &pUnk ) ) )
   {
      lResult = NGLength;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IText, (void**) &pUnk ) ) )
   {
      lResult = NGText;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IAngle, (void**) &pUnk ) ) )
   {
      lResult = NGAngle;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_ICut, (void**) &pUnk ) ) )
   {
      lResult = NGCut;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IImage, (void**) &pUnk ) ) )
   {
      lResult = NGImage;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IExam, (void**) &pUnk ) ) )
   {
      lResult = NGExam;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IPolyline, (void**) &pUnk ) ) )
   {
      lResult = NGPolyline;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IRect, (void**) &pUnk ) ) )
   {
      lResult = NGRect;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_IEllipse, (void**) &pUnk ) ) )
   {
      lResult = NGEllipse;
      pUnk->Release();
   }
   else if( SUCCEEDED( pdisp->QueryInterface( IID_ISymbol, (void**) &pUnk ) ) )
   {
      lResult = NGSymbol;
      pUnk->Release();
   }
   else
      lResult = NGUnknown;

   return lResult;
}

CString CPluginDialog::GetLanguage( long lLCID )
{
   CString strAppLanguage;

   switch ( lLCID )
   {
      case LANGUAGE_GERMAN:
         strAppLanguage = _T("German");
         break;

      case LANGUAGE_ENGLISH_US:
         strAppLanguage = _T("English U.S.");
         break;

      case LANGUAGE_FRENCH:
         strAppLanguage = _T("French");
         break;

      case LANGUAGE_SPANISH:
         strAppLanguage = _T("Spanish");
         break;

      case LANGUAGE_ITALIAN:
         strAppLanguage = _T("Italian");
         break;

      case LANGUAGE_DUTCH:
         strAppLanguage = _T("Dutch");
         break;

      case LANGUAGE_RUSSIAN:
         strAppLanguage = _T("Russian");
         break;

      default:
         break;
   }

   return strAppLanguage;
}

void CPluginDialog::OnUpdate() 
{
   UpdateModel();
}

void CPluginDialog::OnAbout() 
{
   CAboutDlg dlg;
   (void) dlg.DoModal();
}
